/*
 * Copyright (c) 2021 NIBIO <http://www.nibio.no/>. 
 * 
 * This file is part of VIPSLogic.
 * VIPSLogic is free software: you can redistribute it and/or modify
 * it under the terms of the NIBIO Open Source License as published by 
 * NIBIO, either version 1 of the License, or (at your option) any
 * later version.
 * 
 * VIPSLogic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * NIBIO Open Source License for more details.
 * 
 * You should have received a copy of the NIBIO Open Source License
 * along with VIPSLogic.  If not, see <http://www.nibio.no/licenses/>.
 * 
 */

package no.nibio.vips.logic;

import java.util.Set;
import javax.ws.rs.ApplicationPath;
import javax.ws.rs.core.Application;

/**
 * Responsible for adding REST resources
 * @copyright 2016-2021 {@link http://www.nibio.no NIBIO}
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */
@ApplicationPath("/rest/")
public class VIPSLogicApplication extends Application
{
    @Override
    public Set<Class<?>> getClasses() {
        Set<Class<?>> resources = new java.util.HashSet<>();
        //addRestResourceClasses(resources);
        addRestResourceClassesManually(resources);
        return resources;
    }

    /**
     * Handmade because NetBeans doesn't discover the services that are implementations
     * of service interfaces. And it adds the interfaces it finds, which do not work
     * @param resources 
     */
    private void addRestResourceClassesManually(Set<Class<?>> resources) {
        resources.add(no.nibio.vips.logic.service.LogicService.class);
        resources.add(no.nibio.vips.logic.service.POIService.class);
        resources.add(no.nibio.vips.logic.service.AuthenticationService.class);
        resources.add(no.nibio.vips.logic.service.VIPSMobileService.class);
        resources.add(no.nibio.vips.logic.modules.barleynetblotch.BarleyNetBlotchModelService.class);
        resources.add(no.nibio.vips.logic.modules.roughage.RoughageService.class);
        resources.add(no.nibio.vips.logic.modules.barkbeetle.BarkbeetleService.class);
        resources.add(no.nibio.vips.observationdata.ObservationDataService.class);
        resources.add(no.nibio.vips.logic.messaging.sms.SMSHandlingService.class);
        resources.add(no.nibio.vips.logic.modules.applefruitmoth.AppleFruitMothService.class);
        resources.add(no.nibio.vips.logic.service.ObservationService.class);
        resources.add(no.nibio.vips.logic.service.ModelFormService.class);
        
        resources.add(no.nibio.vips.logic.service.JacksonConfig.class);
        //resources.add(no.nibio.vips.logic.service.JSONBConfig.class);
        //resources.add(no.nibio.vips.coremanager.service.ManagerResourceImpl.class);
    }
    
    /**
     * Do not modify addRestResourceClasses() method.
     * It is automatically re-generated by NetBeans REST support to populate
     * given list with all resources defined in the project.
     */
    private void addRestResourceClasses(Set<Class<?>> resources) {
        resources.add(com.fasterxml.jackson.jaxrs.json.JacksonJaxbJsonProvider.class);
        resources.add(com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider.class);
        resources.add(com.webcohesion.enunciate.rt.EnunciateJaxbContextResolver.class);
        resources.add(no.nibio.vips.logic.messaging.sms.SMSHandlingService.class);
        resources.add(no.nibio.vips.logic.modules.applefruitmoth.AppleFruitMothService.class);
        resources.add(no.nibio.vips.logic.modules.barkbeetle.BarkbeetleService.class);
        resources.add(no.nibio.vips.logic.modules.barleynetblotch.BarleyNetBlotchModelService.class);
        resources.add(no.nibio.vips.logic.modules.roughage.RoughageService.class);
        resources.add(no.nibio.vips.logic.service.AuthenticationService.class);
        resources.add(no.nibio.vips.logic.service.JSONBConfig.class);
        resources.add(no.nibio.vips.logic.service.JacksonConfig.class);
        resources.add(no.nibio.vips.logic.service.LogicService.class);
        resources.add(no.nibio.vips.logic.service.ModelFormService.class);
        resources.add(no.nibio.vips.logic.service.ObservationService.class);
        resources.add(no.nibio.vips.logic.service.POIService.class);
        resources.add(no.nibio.vips.logic.service.VIPSMobileService.class);
        resources.add(no.nibio.vips.observationdata.ObservationDataService.class);
        resources.add(org.jboss.resteasy.core.AcceptHeaderByFileSuffixFilter.class);
        resources.add(org.jboss.resteasy.core.AsynchronousDispatcher.class);
        resources.add(org.jboss.resteasy.plugins.interceptors.AcceptEncodingGZIPFilter.class);
        resources.add(org.jboss.resteasy.plugins.interceptors.GZIPDecodingInterceptor.class);
        resources.add(org.jboss.resteasy.plugins.interceptors.GZIPEncodingInterceptor.class);
        resources.add(org.jboss.resteasy.plugins.interceptors.MessageSanitizerContainerResponseFilter.class);
        resources.add(org.jboss.resteasy.plugins.providers.AsyncStreamingOutputProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.ByteArrayProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.DataSourceProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.DefaultBooleanWriter.class);
        resources.add(org.jboss.resteasy.plugins.providers.DefaultNumberWriter.class);
        resources.add(org.jboss.resteasy.plugins.providers.DefaultTextPlain.class);
        resources.add(org.jboss.resteasy.plugins.providers.DocumentProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.FileProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.FileRangeWriter.class);
        resources.add(org.jboss.resteasy.plugins.providers.FormUrlEncodedProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.IIOImageProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.InputStreamProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.JaxrsFormProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.JaxrsServerFormUrlEncodedProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.MultiValuedParamConverterProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.ReaderProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.SourceProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.StreamingOutputProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.StringTextStar.class);
        resources.add(org.jboss.resteasy.plugins.providers.jackson.Jackson2JsonpInterceptor.class);
        resources.add(org.jboss.resteasy.plugins.providers.jackson.PatchMethodFilter.class);
        resources.add(org.jboss.resteasy.plugins.providers.jackson.ResteasyJackson2Provider.class);
        resources.add(org.jboss.resteasy.plugins.providers.jackson.UnrecognizedPropertyExceptionHandler.class);
        resources.add(org.jboss.resteasy.plugins.providers.sse.SseEventProvider.class);
        resources.add(org.jboss.resteasy.plugins.providers.sse.SseEventSinkInterceptor.class);
    }
}